/*
 * tab_font.hpp
 * 
 * This file is part of Knights.
 *
 * Copyright (C) Stephen Thompson, 2006 - 2009.
 * Copyright (C) Kalle Marjola, 1994.
 *
 * Knights is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Knights is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Knights.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * Modifies a Font to draw with tab stops. Used to implement a sort of
 * poor man's multi column list box (Guichan doesn't include a multi
 * column list widget).
 *
 */

#ifndef TAB_FONT_HPP
#define TAB_FONT_HPP

#include "gfx/font.hpp"

#include "boost/shared_ptr.hpp"

#include <vector>

class TabFont : public Coercri::Font {
public:
    TabFont(boost::shared_ptr<Coercri::Font> base_font, const std::vector<int> & field_widths_);
    virtual void drawText(Coercri::GfxContext &dest, int x, int y, const std::string &text,
                          Coercri::Color col, bool antialias) const;
    virtual int getTextHeight() const { return base_font->getTextHeight(); }
    virtual void getTextSize(const std::string &text, int &w, int &h) const;
private:
    boost::shared_ptr<Coercri::Font> base_font;
    std::vector<int> field_widths;
};

#endif
